from rest_framework import generics, status, permissions
from rest_framework.response import Response

from django_resaas.core.utils.translate import Translate
from django_resaas.models.user_login import UserLogin
from django_resaas.data.user.serializers.login import LoginSerializer


class LoginsAPIView(generics.GenericAPIView):
    permission_classes = (permissions.IsAuthenticated,)

    def get(self, request):
        user_login = UserLogin.objects.filter(
            user=request.user
        ).order_by('data', '-hora')

        user_logins = LoginSerializer(
            user_login,
            many=True
        )

        return Response(
            {
                'alert_success': Translate.tdc(
                    request,
                    'Histórico de logins carregado com sucesso'
                ),
                'data': user_logins.data
            },
            status=status.HTTP_200_OK
        )




from django.conf import settings
from django.http import JsonResponse

from rest_framework import status

from django_resaas.core.utils.full_path import FullPath
from django_resaas.core.utils.translate import Translate



class FileAccessMiddleware:

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):

        if request.path.startswith(settings.MEDIA_URL):

            token = request.GET.get('token')

            if token and FullPath.validate_token(token):
                return self.get_response(request)

            return JsonResponse(
                {
                    'alert_error': Translate.tdc(request, 'Nao autorizado')
                },
                status=status.HTTP_401_UNAUTHORIZED
            )

        return self.get_response(request)






from rest_framework import serializers
from rest_framework.exceptions import AuthenticationFailed
from rest_framework_simplejwt.tokens import RefreshToken

from django_resaas.core.utils.translate import Translate


class LogoutSerializer(serializers.Serializer):
    refresh = serializers.CharField()

    def validate(self, attrs):
        self.request = self.context.get('request')
        self.token = attrs.get('refresh')
        return attrs

    def save(self, **kwargs):
        try:
            RefreshToken(self.token).blacklist()
        except Exception:
            raise AuthenticationFailed(
                Translate.tdc(
                    self.request,
                    'Token inválido ou expirado'
                )
            )



# =========================
# Python standard library
# =========================
import importlib


# =========================
# Django
# =========================
from django.apps import apps
from django.core.cache import cache


# =========================
# Django REST Framework
# =========================
from rest_framework import filters
from rest_framework import status
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response


# =========================
# Local application
# =========================
from django_resaas.models.idioma import Idioma
from django_resaas.models.traducao import Traducao
from django_resaas.data.idioma.serializers.idioma import IdiomaSerializer


class IdiomaAPIView(viewsets.ModelViewSet):
    search_fields = ["id", "nome"]
    filter_backends = (filters.SearchFilter,)
    serializer_class = IdiomaSerializer
    queryset = Idioma.objects.all()
    lookup_field = "id"
    pagination_class = None

    def get_queryset(self):
        return self.queryset.order_by("nome")

    def retrieve(self, request, *args, **kwargs):
        idioma = self.get_object()
        serializer = self.get_serializer(idioma)
        return Response(serializer.data, status=status.HTTP_200_OK)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()
        self.perform_destroy(instance)
        return Response(status=status.HTTP_204_NO_CONTENT)

    def update(self, request, *args, **kwargs):
        instance = self.get_object()
        serializer = self.get_serializer(
            instance,
            data=request.data,
            partial=True,
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()

        response = serializer.data
        response["alert_success"] = (
            f"%-{response['nome']}-% foi actualizado com sucesso"
        )

        return Response(response, status=status.HTTP_200_OK)

    def create(self, request, *args, **kwargs):
        data = request.data.copy()
        data["admin"] = request.user.id

        serializer = self.get_serializer(data=data)
        serializer.is_valid(raise_exception=True)
        serializer.save()

        response = serializer.data
        response["alert_success"] = (
            f"%-{response['nome']}-% foi criado com sucesso"
        )

        return Response(response, status=status.HTTP_201_CREATED)

    @action(detail=True, methods=["GET"])
    def traducaos(self, request, *args, **kwargs):
        CACHE_TIMEOUT = 1 * 20  # 1 hora

        idioma = self.get_object()
        lang_code = str(idioma.code).lower().replace("-", "")
        cache_key = f"traducao:{lang_code}"

        cached = cache.get(cache_key)
        if cached is not None:
            return Response(cached, status=status.HTTP_200_OK)

        traducoes = {}

        # Base de dados
        for item in Traducao.objects.filter(idioma_id=idioma.id):
            traducoes[item.chave] = item.traducao

        # Módulos lang/*.py
        for app in apps.get_app_configs():
            module_name = f"{app.name}.lang.{lang_code}"

            try:
                modulo = importlib.import_module(module_name)
            except ModuleNotFoundError:
                continue

            if hasattr(modulo, "key_value"):
                traducoes.update(modulo.key_value)

        cache.set(cache_key, traducoes, CACHE_TIMEOUT)

        return Response(traducoes, status=status.HTTP_200_OK)



from django.db import models
from django_resaas.core.base.models import TimeModel

class CorsAllowedOrigin(TimeModel):
    origin = models.URLField(unique=True)
    def __str__(self):
        return self.origin